#!/usr/bin/env python
# -*- coding: utf-8 -*-
# License:: GPL
# Author:: Alberto Milone (aka tseliot) (mailto:albertomilone@alice.it)
# Website:: http://albertomilone.com

import os, sys, commands, string
import fileinput, re
from subprocess import Popen, PIPE
import classes
import interfaceclasses
import objects

newmethodos = ['feisty', 'gutsy']

def envydependencies():
    '''
    If a user manages to run Envy without installing its dependencies, this function will try to install them.
    If the missing dependencies cannot be installed Envy will exit.
    '''
    dep = classes.Shutilities()
    packages = ['python', 'python-gtk2', 'python-glade2', 'python-vte', 'build-essential', 'xserver-xorg-dev', 'pkg-config', 'gcc', \
    'module-assistant', 'fakeroot', 'dh-make', 'debhelper', 'debconf', 'libstdc++5', 'gksu', 'dpkg-dev', 'bash', 'dash', 'dpatch', \
    'lsb-release']
    dep.checkntry(packages)

def codename():
    """
    Get system codename, can be warty/hoary/breezy/dapper/edgy/etch
    """
    p1 = Popen(['lsb_release', '-c'], stdout=PIPE)
    p = p1.communicate()[0]
    try:
        codename = p.strip()
        tokens = codename.split("\t")
    except AttributeError:
        print 'ERROR: please install lsb-release'
        sys.exit()
    osname = tokens[-1].lower()
    return osname

osname = codename()

def addmakefile32():
    '''Add a makefile.cpu for 32bit recompiled kernels'''
    kernelv = os.uname()[2]
    makefilepath = '/usr/src/' + 'kernel-headers-' + kernelv + '/arch/i386/Makefile.cpu'
    make32 = open(makefilepath, 'w')
    make32.write('# CPU tuning section - shared with UML.\n\
# Must change only cflags-y (or [yn]), not CFLAGS! That makes a difference for UML.\n\
#-mtune exists since gcc 3.4\n\
HAS_MTUNE	:= $(call cc-option-yn, -mtune=i386)\n\
ifeq ($(HAS_MTUNE),y)\n\
tune		= $(call cc-option,-mtune=$(1),)\n\
else\n\
tune		= $(call cc-option,-mcpu=$(1),)\n\
endif\n\
\n\
align := $(cc-option-align)\n\
cflags-$(CONFIG_M386)		+= -march=i386\n\
cflags-$(CONFIG_M486)		+= -march=i486\n\
cflags-$(CONFIG_M586)		+= -march=i586\n\
cflags-$(CONFIG_M586TSC)	+= -march=i586\n\
cflags-$(CONFIG_M586MMX)	+= -march=pentium-mmx\n\
cflags-$(CONFIG_M686)		+= -march=i686\n\
cflags-$(CONFIG_MPENTIUMII)	+= -march=i686 $(call tune,pentium2)\n\
cflags-$(CONFIG_MPENTIUMIII)	+= -march=i686 $(call tune,pentium3)\n\
cflags-$(CONFIG_MPENTIUMM)	+= -march=i686 $(call tune,pentium3)\n\
cflags-$(CONFIG_MPENTIUM4)	+= -march=i686 $(call tune,pentium4)\n\
cflags-$(CONFIG_MK6)		+= -march=k6\n\
# Please note, that patches that add -march=athlon-xp and friends are pointless.\n\
# They make zero difference whatsosever to performance at this time.\n\
cflags-$(CONFIG_MK7)		+= -march=athlon\n\
cflags-$(CONFIG_MK8)		+= $(call cc-option,-march=k8,-march=athlon)\n\
cflags-$(CONFIG_MCRUSOE)	+= -march=i686 $(align)-functions=0 $(align)-jumps=0 $(align)-loops=0\n\
cflags-$(CONFIG_MEFFICEON)	+= -march=i686 $(call tune,pentium3) $(align)-functions=0 $(align)-jumps=0 $(align)-loops=0\n\
cflags-$(CONFIG_MWINCHIPC6)	+= $(call cc-option,-march=winchip-c6,-march=i586)\n\
cflags-$(CONFIG_MWINCHIP2)	+= $(call cc-option,-march=winchip2,-march=i586)\n\
cflags-$(CONFIG_MWINCHIP3D)	+= $(call cc-option,-march=winchip2,-march=i586)\n\
cflags-$(CONFIG_MCYRIXIII)	+= $(call cc-option,-march=c3,-march=i486) $(align)-functions=0 $(align)-jumps=0 $(align)-loops=0\n\
cflags-$(CONFIG_MVIAC3_2)	+= $(call cc-option,-march=c3-2,-march=i686)\n\
\n\
# AMD Elan support\n\
cflags-$(CONFIG_X86_ELAN)	+= -march=i486\n\
\n\
# Geode GX1 support\n\
cflags-$(CONFIG_MGEODEGX1)	+= -march=pentium-mmx\n\
\n\
# add at the end to overwrite eventual tuning options from earlier\n\
# cpu entries\n\
cflags-$(CONFIG_X86_GENERIC) 	+= $(call tune,generic)\n')
    make32.close()

def addmakefile64():
    '''Add a makefile.cpu for 64bit recompiled kernels'''
    kernelv = os.uname()[2]
    makefilepath = '/usr/src/' + 'kernel-headers-' + kernelv + '/arch/i386/Makefile.cpu'
    make64 = open(makefilepath, 'w')
    make64.write('# CPU tuning section - shared with UML.\n\
# Must change only cflags-y (or [yn]), not CFLAGS! That makes a difference for UML.\n\
#-mtune exists since gcc 3.4, and some -mcpu flavors didn\'t exist in gcc 2.95.\n\
HAS_MTUNE	:= $(call cc-option-yn, -mtune=i386)\n\
ifeq ($(HAS_MTUNE),y)\n\
tune		= $(call cc-option,-mtune=$(1),)\n\
else\n\
tune		= $(call cc-option,-mcpu=$(1),)\n\
endif\n\
\n\
align := $(cc-option-align)\n\
cflags-$(CONFIG_M386)		+= -march=i386\n\
cflags-$(CONFIG_M486)		+= -march=i486\n\
cflags-$(CONFIG_M586)		+= -march=i586\n\
cflags-$(CONFIG_M586TSC)	+= -march=i586\n\
cflags-$(CONFIG_M586MMX)	+= $(call cc-option,-march=pentium-mmx,-march=i586)\n\
cflags-$(CONFIG_M686)		+= -march=i686\n\
cflags-$(CONFIG_MPENTIUMII)	+= -march=i686 $(call tune,pentium2)\n\
cflags-$(CONFIG_MPENTIUMIII)	+= -march=i686 $(call tune,pentium3)\n\
cflags-$(CONFIG_MPENTIUMM)	+= -march=i686 $(call tune,pentium3)\n\
cflags-$(CONFIG_MPENTIUM4)	+= -march=i686 $(call tune,pentium4)\n\
cflags-$(CONFIG_MK6)		+= -march=k6\n\
# Please note, that patches that add -march=athlon-xp and friends are pointless.\n\
# They make zero difference whatsosever to performance at this time.\n\
cflags-$(CONFIG_MK7)		+= $(call cc-option,-march=athlon,-march=i686 $(align)-functions=4)\n\
cflags-$(CONFIG_MK8)		+= $(call cc-option,-march=k8,$(call cc-option,-march=athlon,-march=i686 $(align)-functions=4))\n\
cflags-$(CONFIG_MCRUSOE)	+= -march=i686 $(align)-functions=0 $(align)-jumps=0 $(align)-loops=0\n\
cflags-$(CONFIG_MEFFICEON)	+= -march=i686 $(call tune,pentium3) $(align)-functions=0 $(align)-jumps=0 $(align)-loops=0\n\
cflags-$(CONFIG_MWINCHIPC6)	+= $(call cc-option,-march=winchip-c6,-march=i586)\n\
cflags-$(CONFIG_MWINCHIP2)	+= $(call cc-option,-march=winchip2,-march=i586)\n\
cflags-$(CONFIG_MWINCHIP3D)	+= $(call cc-option,-march=winchip2,-march=i586)\n\
cflags-$(CONFIG_MCYRIXIII)	+= $(call cc-option,-march=c3,-march=i486) $(align)-functions=0 $(align)-jumps=0 $(align)-loops=0\n\
cflags-$(CONFIG_MVIAC3_2)	+= $(call cc-option,-march=c3-2,-march=i686)\n\
\n\
# AMD Elan support\n\
cflags-$(CONFIG_X86_ELAN)	+= -march=i486\n\
\n\
# Geode GX1 support\n\
cflags-$(CONFIG_MGEODEGX1)		+= $(call cc-option,-march=pentium-mmx,-march=i486)\n')
    make64.close()

def rmmakefile():
    '''Remove the makefile.cpu for recompiled kernels'''
    kernelv = os.uname()[2]
    makefilepath = '/usr/src/' + 'kernel-headers-' + kernelv + '/arch/i386/Makefile.cpu'
    try:
        os.remove(makefilepath)
    except (OSError):
        pass


def removediversion(file):
    check = classes.Shutilities()
    diversions = []
    divert1 = Popen(['dpkg-divert', '--list'], stdout=PIPE)
    divert2 = Popen(['grep', file], stdin=divert1.stdout, stdout=PIPE)
    divout = divert2.communicate()[0]
    output = divout.split('\n')
    #print 'output is ' + str(output)
    #print 'lenoutput is ' + str(len(output))
    if output[0] != '':
        for line in output:
            try:
                diversions.append(line.split(' ')[2])
            except IndexError:
                pass
        if len(diversions) != 0:
            for i in diversions:
                check.cmdoutput( 'sudo dpkg-divert --rename --remove ' + i, 'noverbose')

def glxdiversion():
    removediversion('libglx.so')
    removediversion('libGL')
 
def remove_nvidia_pkg():
    '''
    Solo per il metodo vecchio.
    
    Per quello nuovo servono
    libgl1-mesa-dev libglu1-mesa-dev mesa-common-dev
    
    '''
    if osname in newmethodos:#NEW METHOD
        packages = ['nvidia-glx', 'nvidia-kernel-`uname -r`', 'nvidia-glx-dev', 'nvidia-kernel-source', \
       'nvidia-glx', 'nvidia-kernel-`uname -r`', 'nvidia-glx-dev', 'nvidia-kernel-source', 'nvidia-glx-new', \
       'nvidia-new-kernel-`uname -r`', 'nvidia-glx-new-dev', 'nvidia-new-kernel-source',  \
       'nvidia-glx-legacy', 'nvidia-legacy-kernel-source', 'nvidia-settings', 'nvidia-xconfig']
    else:#LEGACY METHOD
        packages = ['nvidia-glx', 'nvidia-kernel-`uname -r`', 'nvidia-glx-dev', 'nvidia-kernel-source', \
       'nvidia-glx', 'nvidia-kernel-`uname -r`', 'nvidia-glx-dev', 'nvidia-kernel-source', 'nvidia-glx-new', \
       'nvidia-new-kernel-`uname -r`', 'nvidia-glx-new-dev', 'nvidia-new-kernel-source', 'libgl1-mesa-dev', \
       'mesa-common-dev', 'nvidia-glx-legacy', 'nvidia-legacy-kernel-source', 'nvidia-settings', 'nvidia-xconfig']
    check = classes.Shutilities()
    check.pkgremove(packages, 'purge')
    files = ['/usr/src/nvidia*.deb', '/usr/lib/xorg/modules/libglx.so']
    dirs = ['/usr/src/modules/nvidia*']
    check.removefiles(files, 'noverbose')
    check.removedirs(dirs, 'noverbose')
##    check.cmdoutput('sudo apt-get --assume-yes  --purge remove nvidia-glx nvidia-kernel-`uname -r` nvidia-glx-dev nvidia-kernel-source 1>&2')
##    check.cmdoutput('sudo apt-get --assume-yes  --purge remove nvidia-glx-new nvidia-new-kernel-`uname -r` nvidia-glx-new-dev 1>&2')
##    check.cmdoutput('sudo apt-get --assume-yes  --purge nvidia-new-kernel-source libgl1-mesa-dev mesa-common-dev 1>&2')
##    check.cmdoutput('sudo apt-get --assume-yes  --purge remove nvidia-glx-legacy nvidia-legacy-kernel-source 1>&2')
##    check.cmdoutput('sudo apt-get --assume-yes  --purge remove nvidia-settings nvidia-xconfig')
##    check.cmdoutput('sudo rm /usr/src/nvidia*.deb 2>/dev/null')
##    check.cmdoutput('sudo rm -R /usr/src/modules/nvidia* 2>/dev/null')


def remove_ati_pkg():
    packages = ['fglrx-control', 'fglrx-kernel-`uname -r`', 'fglrx-kernel-source', \
    'xorg-driver-fglrx', 'xorg-driver-fglrx-dev', 'fglrx-amdcccle']
    check = classes.Shutilities()
    check.pkgremove(packages, 'purge')
    dirs = ['/usr/src/fglrx*.deb']
    files = ['/usr/src/modules/fglrx*']
    check.removefiles(files, 'noverbose')
    check.removedirs(dirs, 'noverbose')
    
##    check.cmdoutput('sudo apt-get --assume-yes  --purge remove fglrx-control fglrx-kernel-`uname -r` fglrx-kernel-source xorg-driver-fglrx xorg-driver-fglrx-dev fglrx-amdcccle 1>&2')
##    check.cmdoutput('sudo rm /usr/src/fglrx*.deb 2>/dev/null')
##    check.cmdoutput('sudo rm -R /usr/src/modules/fglrx* 2>/dev/null')

def dm_stop():
    check = classes.Shutilities()
    p1 = Popen(['ls', '/etc/init.d/'], stdout=PIPE)
    p = p1.communicate()[0]
    c = p.split('\n')
    if 'gdm' in c:
        print 'GDM detected'
        check.cmdoutput('sudo /etc/init.d/gdm stop', 'noverbose')
    elif kdm in c:
        print 'KDM detected'
        check.cmdoutput('sudo /etc/init.d/kdm stop', 'noverbose')
    else:
        print 'ENVY: No known desktop manager was found.\nPlease stop the xserver manually'
        sys.exit()

def remove_deb():#for both ATI and Nvidia
    check = classes.Shutilities()
    files = ['*.deb']
    check.removefiles(files, 'noverbose')

def atiremovechanges():#removes files such as fglrx-installer_8.33.6-1_i386.changes
    check = classes.Shutilities()
    files = ['*.changes']
    check.removefiles(files, 'noverbose')
    
def nvremovechanges():#removes files such as fglrx-installer_8.33.6-1_i386.changes
    check = classes.Shutilities()
    files = ['*.changes', '*.asc']
    check.removefiles(files, 'noverbose')

def remove_debsrc():#for both ATI and Nvidia
##    check.cmdoutput('/usr/src/*.deb')
##    check.cmdoutput('sudo rm -R /usr/src/modules/nvidia-*')
##    check.cmdoutput('sudo rm -R /usr/src/modules/fglrx-kernel')
    check = classes.Shutilities()
    dirs = ['/usr/src/modules/nvidia-*', '/usr/src/modules/fglrx-kernel']
    files = ['/usr/src/*.deb']
    check.removefiles(files, 'noverbose')
    check.removedirs(dirs, 'noverbose')

def depmodules():
    check = classes.Shutilities()
    check.cmdoutput('sudo depmod -ae')

def restoremesa():
    check = classes.Shutilities()
    check.cmdoutput('sudo apt-get -y install --reinstall libgl1-mesa-glx libglu1-mesa  libgl1-mesa-dri')

def pcrestart():
    check = classes.Shutilities()
    while 1:
        xsrv = raw_input('Do you want to restart your computer now (Recommended)? (y/n) \ "y" is the default answer\n').strip().lower()
        if xsrv == 'y' or xsrv == '' or xsrv == 'yes':
            check.cmdoutput('sudo reboot')
            break
            sys.exit()
        elif xsrv == 'n':
            print 'Remember to restart your computer manually'
            break
            sys.exit()
        elif xsrv == 'no':
            print 'Remember to restart your computer manually'
            break
            sys.exit()

def dependencies():
    check = classes.Shutilities()
    check.pkginstall('linux-headers-`uname -r`')
    #check.cmdoutput('sudo apt-get --assume-yes  install linux-headers-`uname -r`')
    #check.cmdoutput('sudo apt-get --assume-yes  install build-essential')
    #check.cmdoutput('sudo apt-get --assume-yes  install xserver-xorg-dev')
    #check.cmdoutput('sudo apt-get --assume-yes  install pkg-config')
    #check.cmdoutput('sudo apt-get --assume-yes  install gcc')
    #check.cmdoutput('sudo apt-get --assume-yes  install module-assistant build-essential fakeroot dh-make debhelper debconf libstdc++5')

def newdependencies():
##    os.system('sudo apt-get --assume-yes  install cpio debhelper libqt3-mt-dev kernel-wedge module-init-tools rpm sed sharutils bzip2 libgtk2.0-dev libxxf86misc-dev libxtst-dev libxxf86vm-dev libxinerama-dev libstdc++5')
##    #libqt3-mt-dev kernel-wedge sharutils libgtk2.0-dev libxxf86misc-dev libxtst-dev libxxf86vm-dev libxinerama-dev
##    if os.uname()[-1] == 'x86_64':
##         os.system('sudo apt-get --assume-yes  install libc6-i386')
    if osname in newmethodos:#Only for the new method with Nvidia cards
        packagestocheck = ['cpio', 'debhelper', 'libqt3-mt-dev', 'kernel-wedge', 'module-init-tools', 'rpm', 'sed', 'sharutils', 'bzip2', 'libgtk2.0-dev', 'libxxf86misc-dev', 'libxtst-dev', 'libxxf86vm-dev', 'libxinerama-dev', 'libstdc++5']
        print 'Checking the Dependencies for the New Method'
        check = classes.Shutilities()
        check.checkntry(packagestocheck)
        if os.uname()[-1] == 'x86_64':
            check.checkntry('libc6-i386')


    
 
 
 
def blkresmanager():
##    os.system('sudo rm /lib/linux-restricted-modules/.nvidia* 2>/dev/null')
##    os.system('sudo rm /lib/modules/`uname -r`/volatile/nvidia* 2>/dev/null')#nvidia_new.ko can cause problems of API mismatch
    check = classes.Shutilities()
    files = ['/lib/linux-restricted-modules/.nvidia*', '/lib/modules/`uname -r`/volatile/nvidia*']
    check.removefiles(files, 'noverbose')

def xserverstart():
    check = classes.Shutilities()
    while 1:
        xsrv = raw_input('Do you want to Start the Xserver now? (y/n) \ "y" is the default answer\n').strip().lower()
        if xsrv == 'y' or xsrv == '' or xsrv == 'yes':
            p1 = Popen(['ls', '/etc/init.d/'], stdout=PIPE)
            p = p1.communicate()[0]
            c = p.split('\n')
            if 'gdm' in c:
                print 'GDM detected'
                check.cmdoutput('sudo /etc/init.d/gdm restart', 'noverbose')
            elif kdm in c:
                print 'KDM detected'
                check.cmdoutput('sudo /etc/init.d/kdm restart', 'noverbose')
            else:
                print 'ENVY: No known desktop manager was found.\nPlease restart the xserver manually'
            #os.system('sudo /etc/init.d/gdm restart; sudo /etc/init.d/kdm restart')
            sys.exit()
            break
        elif xsrv == 'n':
            print 'Remember to start the xserver manually'
            break
            #sys.exit()
        elif xsrv.lower() == 'no':
            print 'Remember to start the xserver manually'
            break
            #sys.exit()


def xorgset():
    answer = raw_input('Do you want your xorg.conf to be automatically configured? (y/n) \ "y" is the default answer\n')
    while 1:
        if answer == 'y' or answer == '':
            return 'y'
        elif answer == 'n':
            return 'n'





